require 'spec_helper'

describe PlanBCD::Rails::Middleware do
  let(:app) { double call: [200, {'Content-Type' => 'text/html; charset=utf-8'}, ['<html><head>something</head><body></body></html>']] }
  let(:js_url) { '//planb.cd/test.js' }
  subject { PlanBCD::Rails::Middleware.new(app) }
  before do
    PlanBCD::Rails.configure do |c|
      c.js_url = js_url
      c.auto_insert = true
    end
  end

  it "inserts planbcd tag" do
    status, headers, body = subject.call({})
    expect(status).to eq(200)
    expect(headers).to eq({'Content-Type' => 'text/html; charset=utf-8'})
    expect(body).to eq([%Q~<html><head><script type=\"text/javascript\">(function(){!function(a,b,c,d,e){return d=\"http\"+(\"https:\"===c?\"s\":\"\")+\":\"+d,a.planBCDObject=e,a[e]||(a[e]=function(){var b;return((b=a[e]).q||(b.q=[])).push(arguments),a[e].l=1*new Date}),b.write(unescape('%3Cscript type=\"text/javascript\" src=\"'+d+'\"%3E%3C/script%3E'))}(window,document,document.location.protocol,\"//planb.cd/test.js\",\"pbcd\")}).call(this);</script>something</head><body></body></html>~])
  end
  context "head tag with something" do
    let(:app) { double call: [200, {'Content-Type' => 'text/html; charset=utf-8'}, ['<html><head abcdefg>something</head><body></body></html>']] }
    it "inserts planbcd tag" do
      status, headers, body = subject.call({})
      expect(status).to eq(200)
      expect(headers).to eq({'Content-Type' => 'text/html; charset=utf-8'})
      expect(body).to eq([%Q~<html><head abcdefg><script type=\"text/javascript\">(function(){!function(a,b,c,d,e){return d=\"http\"+(\"https:\"===c?\"s\":\"\")+\":\"+d,a.planBCDObject=e,a[e]||(a[e]=function(){var b;return((b=a[e]).q||(b.q=[])).push(arguments),a[e].l=1*new Date}),b.write(unescape('%3Cscript type=\"text/javascript\" src=\"'+d+'\"%3E%3C/script%3E'))}(window,document,document.location.protocol,\"//planb.cd/test.js\",\"pbcd\")}).call(this);</script>something</head><body></body></html>~])
    end
  end
  context "no head tag in body" do
    let(:app) { double call: [200, {'Content-Type' => 'text/html; charset=utf-8'}, ['<html><body></body></html>']] }
    it "does not insert planbcd tag if the body does not have head tag" do
      status, headers, body = subject.call({})
      expect(status).to eq(200)
      expect(headers).to eq({'Content-Type' => 'text/html; charset=utf-8'})
      expect(body).to eq([%Q~<html><body></body></html>~])
    end
  end
  context "not html request" do
    let(:app) { double call: [200, {'Content-Type' => 'application/json;'}, ['{}']] }
    it "does not insert planbcd tag if the body does not have head tag" do
      expect(subject).not_to receive(:insert_js)
      status, headers, body = subject.call({})
      expect(status).to eq(200)
      expect(headers).to eq({'Content-Type' => 'application/json;'})
      expect(body).to eq(['{}'])
    end
  end
  context "no js_url" do
    let(:js_url) { nil }
    it "does not raise error but write the log" do
      expect($stderr).to receive(:write).with(/^Failed to insert planbcd tag: /)
      expect {
        subject.call({})
      }.not_to raise_error
    end
  end
end
