require 'spec_helper'

describe PlanBCD::Rails::ViewHelpers do
  include PlanBCD::Rails::ViewHelpers

  describe "#pbcd_init" do
    let(:js_url) { '//planb.cd/test.js' }
    before do
      PlanBCD::Rails.configure do |c|
        c.js_url = js_url
      end
    end
    it "generate planbcd tag" do
      expect(pbcd_init).to eq(%Q~<script type=\"text/javascript\">(function(){!function(a,b,c,d,e){return d=\"http\"+(\"https:\"===c?\"s\":\"\")+\":\"+d,a.planBCDObject=e,a[e]||(a[e]=function(){var b;return((b=a[e]).q||(b.q=[])).push(arguments),a[e].l=1*new Date}),b.write(unescape('%3Cscript type=\"text/javascript\" src=\"'+d+'\"%3E%3C/script%3E'))}(window,document,document.location.protocol,\"//planb.cd/test.js\",\"pbcd\")}).call(this);</script>~)
    end
    context "js_url has scheme" do
      let(:js_url) { 'http://planb.cd/test.js' }
      it "generate planbcd tag without scheme" do
        expect(pbcd_init).to eq(%Q~<script type=\"text/javascript\">(function(){!function(a,b,c,d,e){return d=\"http\"+(\"https:\"===c?\"s\":\"\")+\":\"+d,a.planBCDObject=e,a[e]||(a[e]=function(){var b;return((b=a[e]).q||(b.q=[])).push(arguments),a[e].l=1*new Date}),b.write(unescape('%3Cscript type=\"text/javascript\" src=\"'+d+'\"%3E%3C/script%3E'))}(window,document,document.location.protocol,\"//planb.cd/test.js\",\"pbcd\")}).call(this);</script>~)
      end
    end
    context "js_url is proc" do
      let(:a_controller) { double }
      let(:js_url) { proc { |controller| '//planb.cd/proc.js' } }
      before do
        allow(self).to receive(:controller).and_return(a_controller)
      end
      it "generate planbcd tag" do
        expect(pbcd_init).to eq(%Q~<script type=\"text/javascript\">(function(){!function(a,b,c,d,e){return d=\"http\"+(\"https:\"===c?\"s\":\"\")+\":\"+d,a.planBCDObject=e,a[e]||(a[e]=function(){var b;return((b=a[e]).q||(b.q=[])).push(arguments),a[e].l=1*new Date}),b.write(unescape('%3Cscript type=\"text/javascript\" src=\"'+d+'\"%3E%3C/script%3E'))}(window,document,document.location.protocol,\"//planb.cd/proc.js\",\"pbcd\")}).call(this);</script>~)
      end
    end
    context "js_url is nil" do
      let(:js_url) { nil }
      it "raises error" do
        expect {
          pbcd_init
        }.to raise_error(PlanBCD::Rails::RuntimeError)
      end
    end
    context "js_url is empty" do
      let(:js_url) { '' }
      it "raises error" do
        expect {
          pbcd_init
        }.to raise_error(PlanBCD::Rails::RuntimeError)
      end
    end
  end
end
